package com.adc.da.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import com.adc.da.http.ApiResult;
import com.adc.da.util.PDFUtils;
import com.cqcca.jni.QR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Api(tags = "二维码生成控件")
@Slf4j
public class QRController extends BaseController {

    @Value("${file.tempDir}")
    private String tmpDir;

    @ApiOperation(value = "生成二维码的base64编码，格式为PNG")
    @GetMapping("genQR/{arg1}")
    public ApiResult getGenQR(@PathVariable String arg1) {
        return ApiResult.ok(QR.doGenQR(arg1));
    }

    @ApiOperation(value = "生成加密字符串")
    @GetMapping("encrypt/{arg1}")
    public ApiResult getDESEncrypt(@PathVariable String arg1) {
        return ApiResult.ok(QR.doDESEncrypt(arg1));
    }

    @ApiOperation(value = "验证加密字符串")
    @GetMapping("decrypt/{arg1}")
    public ApiResult getDESDecrypt(@PathVariable String arg1) {
        return ApiResult.ok(QR.doDESDecrypt(arg1));
    }

    @ApiOperation(value = "批量生成二维码的base64编码，格式为PNG")
    @PostMapping("batchGenQR")
    public ApiResult batchGenQR(@RequestBody List<String> args) {
        if (CollectionUtil.isEmpty(args)) {
            return ApiResult.fail("参数不能为空");
        }
        Map<String, String> res = new HashMap<>();
        args.forEach(key -> res.put(key, QR.doGenQR(key)));
        return ApiResult.ok(res);
    }

    @ApiOperation(value = "批量生成加密字符串")
    @PostMapping("batchEncrypt")
    public ApiResult batchDESEncrypt(@RequestBody List<String> args) {
        if (CollectionUtil.isEmpty(args)) {
            return ApiResult.fail("参数不能为空");
        }
        Map<String, String> res = new HashMap<>();
        args.forEach(key -> res.put(key, QR.doDESEncrypt(key)));
        return ApiResult.ok(res);
    }

    @ApiOperation(value = "批量验证加密字符串")
    @PostMapping("batchDecrypt")
    public ApiResult batchDESDecrypt(@RequestBody List<String> args) {
        if (CollectionUtil.isEmpty(args)) {
            return ApiResult.fail("参数不能为空");
        }
        Map<String, String> res = new HashMap<>();
        args.forEach(key -> res.put(key, QR.doDESDecrypt(key)));
        return ApiResult.ok(res);
    }

    @ApiOperation(value = "word转pdf")
    @PostMapping("wordToPdf")
    public ResponseEntity wordToPdf(@RequestParam MultipartFile file) {
        int index = file.getOriginalFilename().lastIndexOf(".docx");
        if (index == -1) {
            return new ResponseEntity("文件格式只支持docx", HttpStatus.BAD_REQUEST);
        }
        String fileName = file.getOriginalFilename().substring(0, index);
        if (!FileUtil.exist(tmpDir)) {
            FileUtil.mkdir(tmpDir);
        }
        String wordFile = tmpDir + File.separator + System.currentTimeMillis() + file.getOriginalFilename();
        String pdfFile = tmpDir + File.separator + System.currentTimeMillis() + fileName + ".pdf";
        try {
            InputStream is = file.getInputStream();
            byte[] buffer = new byte[1024];
            int byteRead;
            FileOutputStream fos = new FileOutputStream(wordFile);
            while ((byteRead = is.read(buffer)) != -1) {
                fos.write(buffer, 0, byteRead);
            }
            fos.close();

            PDFUtils.wordToPDF(wordFile, pdfFile);
            byte[] data = org.apache.commons.io.FileUtils.readFileToByteArray(new File(pdfFile));
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
            headers.setContentDisposition(ContentDisposition.builder("attachment").filename(URLEncoder.encode(fileName, "utf-8")).build());
            return new ResponseEntity(data, headers, HttpStatus.OK);
        } catch (Exception e) {
            log.error("word转换pdf失败", e);
            return new ResponseEntity("word转换pdf失败", HttpStatus.EXPECTATION_FAILED);
        } finally {
            FileUtil.del(wordFile);
            FileUtil.del(pdfFile);
        }
    }
}
